// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDESU_VERSION_H
#define KDESU_VERSION_H

#define KDESU_VERSION_STRING "5.43.0"
#define KDESU_VERSION_MAJOR 5
#define KDESU_VERSION_MINOR 43
#define KDESU_VERSION_PATCH 0
#define KDESU_VERSION ((5<<16)|(43<<8)|(0))

#endif
